/* This file is automatically copied form (NexJ PhoneGap) DevicePlugin project.
 * Do not modify this file -- YOUR CHANGES WILL BE ERASED! 
 */ 
package com.nexj.bluetooth;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.Vector;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.util.Log;

import com.nexj.bluetooth.parser.PacketParser;

public abstract class BluetoothDeviceAdapter extends DeviceManager
{

    protected static List<BluetoothSocket> BLUETOOTH_SOCKET_LIST;

    public static final Map<String, String> BRAND_ADAPTER_CLASS_MAP;
    public static final Map<String, Object> BRAND_PARSER_TYPES_MAP;

    // Service ID required for Bluetooth Serial Port Profile connections
    // Documentation: https://developer.android.com/reference/android/bluetooth/BluetoothDevice.html
    protected static final UUID SERVICE_ID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

    protected final String m_sLogTag;
    protected final PacketParser<?, ?> m_packetParser;

    static
    {
        BRAND_ADAPTER_CLASS_MAP = new HashMap<String, String>(4);
        BRAND_PARSER_TYPES_MAP = new HashMap<String, Object>(4);
        BLUETOOTH_SOCKET_LIST = new Vector<BluetoothSocket>(16);

        // TODO: This is not a scalable option to manage supported devices
        BRAND_ADAPTER_CLASS_MAP.put("AND", ANDBluetoothServer.class.getName());
    }

    public BluetoothDeviceAdapter(PacketParser<?, ?> parser, String sLogTag)
    {
        m_packetParser = parser;

        BRAND_PARSER_TYPES_MAP.put(getDeviceManufacturer(), Arrays.asList(parser.getSupportedDeviceTypes()));

        m_sLogTag = sLogTag;
    }

    public static void stop()
    {
        if (BluetoothAndroid.isSupported())
        {
            BluetoothAndroid.getAdapter().cancelDiscovery();
        }

        Iterator<BluetoothSocket> clientLoop = BLUETOOTH_SOCKET_LIST.iterator();

        while (clientLoop.hasNext())
        {
            BluetoothSocket client = clientLoop.next();

            BLUETOOTH_SOCKET_LIST.remove(client);

            try
            {
                client.close();
            }
            catch (Exception ignored)
            {
                Log.e(BluetoothDeviceAdapter.class.getName(), "Error closing resource: ClientSocket");
            }
        }
    }



    protected abstract String getDeviceManufacturer();

    protected abstract String getDeviceProtocol();

    protected abstract String getDeviceSerial(BluetoothDevice device);

    protected abstract String getDeviceType(BluetoothDevice device);

    public abstract boolean isConnected();

    public abstract Map<String, Object> connect(Map<String, Object> params) throws Exception;

    public abstract void disconnect(Map<String, Object> params);
}