/* This file is automatically copied form (NexJ PhoneGap) DevicePlugin project.
 * Do not modify this file -- YOUR CHANGES WILL BE ERASED! 
 */ 
package com.nexj.bluetooth;

import java.lang.reflect.Field;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;

import com.nexj.pseudo.phonegap.CallbackContext;
import com.nexj.pseudo.phonegap.PluginResult;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import com.nexj.bluetooth.util.ResultFactory;
import com.uhn.BlugluLibDroid.Bluglu;

public class BlugluDeviceManager extends DeviceManager
{
	// Debugging
    private static final String TAG = "BlugluDevoceManager";
    private static final boolean D = false;

    // Message types sent from the Bluglu Handler
    public static final int MESSAGE_STATE_CHANGE = 1;
    public static final int MESSAGE_READ = 2;
    public static final int MESSAGE_WRITE = 3;
    public static final int MESSAGE_DEVICE_NAME = 4;
    public static final int MESSAGE_TOAST = 5;

    // Keys of pass-back payload
    public static final String PAYLOAD_TYPE = "type";
    public static final String PAYLOAD_BRAND = "brand";
    public static final String PAYLOAD_TIME = "time";
    public static final String PAYLOAD_READING = "reading";
    public static final String PAYLOAD_SYSTEM = "system";
    public static final String PAYLOAD_SERIAL = "serial";
    public static final String PAYLOAD_BATTERY_STATUS_PERCENTAGE = "battery status(%)";

	// Key names received from the Bluglu Handler
    public static final String DEVICE_NAME = "device_name";
    public static final String TOAST = "toast";

    //private Handler and working thread for receiving data
    protected Thread m_workingThread;
    protected Handler m_messageHandler;

    // Members for persisting passed-in parameters
    protected final MedicalDevicePlugin m_plugin;

    protected CallbackContext m_callback;
    protected String m_CallbackId; // may be removed
    protected String m_Brand = "";
    protected String m_DeviceName = "";
    protected String m_SerialNumber = "";

    // Member object for the Bluglu service
    private Bluglu m_Bluglu = null;

    // JSON object which holds the data for each reading
    private JSONObject payload;
    // JSON object which holds the state for Bluetooth connection.
    private JSONObject statePayload = new JSONObject();

    public BlugluDeviceManager(MedicalDevicePlugin plugin)
    {
        // If the adapter is null, then Bluetooth is not supported
        if (BluetoothAndroid.BLUETOOTH_ADAPTER == null)
        {
            Log.e("BluetoothDeviceManager", "+++ Bluetooth is not available +++");
        }

        m_plugin = plugin;

        if (D)
            Log.i(TAG, "+++ CREATING BlugluManager +++");
    }

    public void _start(final HashMap<String, Object> params, CallbackContext callback)
    {
        m_Brand = (String)params.get("brand");
        m_DeviceName = (String)params.get("deviceName");
        m_SerialNumber = (String)params.get("serial");
        m_callback = callback;

        m_workingThread = new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                Looper.prepare();

                m_messageHandler = new Handler()
                {
                    @Override
                    public void handleMessage(Message msg)
                    {
                        switch (msg.what)
                        {
                            case MESSAGE_STATE_CHANGE:
                                if (D)
                                    Log.i(TAG, "MESSAGE_STATE_CHANGE: " + msg.arg1);
                                switch (msg.arg1)
                                {
                                    case Bluglu.STATE_CONNECTED:
                                        if (D)
                                            Log.i(TAG, "+++ Gathering Information +++");
                                        // we may provide connection state later.
                                        //updateState("CONNECTED");
                                        break;
                                    case Bluglu.STATE_CONNECTING:
                                        if (D)
                                            Log.i(TAG, "+++ connecting... +++");
                                        //updateState("CONNECTING");
                                        break;
                                    case Bluglu.STATE_LISTEN:
                                        // Reset pass-back data set
                                        payload = new JSONObject();

                                        if (D)
                                            Log.i(TAG, "+++ Listening +++");
                                        //updateState("LISTENING");
                                        break;
                                    case Bluglu.STATE_NONE:
                                        if (D)
                                            Log.i(TAG, "+++ not connected +++");
                                        //updateState("NOT CONNECTED");
                                        break;
                                }
                                break;
                            case MESSAGE_READ:
                                switch (msg.arg1)
                                {
                                    case Bluglu.MESSAGE_BLUGLU_BATTERY_STATUS:
                                        if (D)
                                            Log.d(TAG, "MESSAGE_BLUGLU_BATTERY_STATUS");
                                        Log.i(TAG, "Bluglu Battery Status: " + msg.arg2 + "%");
                                        prepareData(PAYLOAD_BATTERY_STATUS_PERCENTAGE, msg.arg2);
                                        break;
                                    case Bluglu.MESSAGE_BLUGLU_OTUMINI_SERIAL:
                                        String serial = msg.obj.toString();
                                        if (D)
                                            Log.d(TAG, "MESSAGE_BLUGLU_OTUMINI_SERIAL");
                                        Log.i(TAG, "OTUMini Serial Number: " + serial);
                                        if (m_SerialNumber == null || m_SerialNumber.equals("") || m_SerialNumber.equals(serial))
                                        {
                                            prepareData(PAYLOAD_SERIAL, serial);
                                        }
                                        else
                                        {
                                            if (m_Bluglu != null)
                                            {
                                                m_Bluglu.stop();
                                                setupDevice();
                                                m_Bluglu.start();

                                                if (D)
                                                    Log.e(TAG, "++ Device serial number was not matched! Restart Bluetooth server for bluglu device++");
                                                returnResult(ResultFactory.createResult(PluginResult.Status.ERROR,
                                                            "Device serial number was not matched!"));
                                            }
                                        }
                                        break;
                                    case Bluglu.MESSAGE_BLUGLU_OTUMINI_READING:
                                        Object mObj = msg.obj;
                                        Class<?> g = mObj.getClass();
                                        try
                                        {
                                            // get the creation date of the record
                                            Field gDate = g.getField("recordCreationDate");
                                            String gDateString = gDate.get(mObj).toString();
                                            Calendar calendar = Calendar.getInstance();
                                            SimpleDateFormat sdf = new SimpleDateFormat("MMM d, yyyy h:mm:ss a");
                                            try
                                            {
                                                calendar.setTime(sdf.parse(gDateString));
                                            }
                                            catch (ParseException e)
                                            {
                                            }
                                            Log.i(TAG, "Record Creation Date : " + gDateString);
                                            prepareData(PAYLOAD_TIME, calendar.getTimeInMillis() / 1000L);
                                        }
                                        catch (NoSuchFieldException e)
                                        {
                                            returnError("Error when convert " + PAYLOAD_TIME);
                                        }
                                        catch (IllegalAccessException e)
                                        {
                                            returnError("Error when convert " + PAYLOAD_TIME);
                                        }

                                        try
                                        {
                                            // get the value of the record
                                            Field gValue = g.getField("recordValue");
                                            Float gValueFloat = gValue.getFloat(mObj);
                                            //mReadingValue.setText(gValueFloat.toString() + " mmol/L");
                                            Log.i(TAG, "OTUMini Reading : " + gValueFloat.toString() + " mmol/L");
                                            prepareData(PAYLOAD_READING, gValueFloat);
                                        }
                                        catch (NoSuchFieldException e)
                                        {
                                            returnError("Error when convert " + PAYLOAD_TIME);
                                        }
                                        catch (IllegalAccessException e)
                                        {
                                            returnError("Error when convert " + PAYLOAD_TIME);
                                        }
                                        break;
                                }
                                break;
                            case MESSAGE_DEVICE_NAME:
                                String mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
                                Log.i(TAG, "Connected to " + mConnectedDeviceName);
                                prepareData(PAYLOAD_BRAND, mConnectedDeviceName);
                                break;
                            case MESSAGE_TOAST:

                                Log.i(TAG, msg.getData().getString(TOAST));
                                break;
                        }
                    }
                };

                // If BT is not on, request that it be enabled.
                if (!BluetoothAndroid.isEnabled())
                {
                    returnError("Bluetooth is not enabled");
                }
                else
                {
                    if (m_Bluglu == null)
                        setupDevice();
                    if (D)
                        Log.i(TAG, "++ Setup Bluglu ++");
                }

                if (m_Bluglu != null)
                {
                    // Only if the state is STATE_NONE, do we know that we haven't started already
                    if (m_Bluglu.getState() == Bluglu.STATE_NONE)
                    {
                        // Start the Bluglu service
                        m_Bluglu.start();

                        if (D)
                            Log.i(TAG, "++ Bluglu started! ++");
                    }
                }

                Looper.loop();
            }
        });

        m_workingThread.start();
    }

    private void setupDevice()
    {
        Log.d(TAG, "setupDevice()");

        // Initialize Bluglu service
        m_Bluglu = new Bluglu((Context)m_plugin.m_cordova, m_messageHandler);
    }

    @Override
    public void _stop(CallbackContext callback)
    {
        if (D)
            Log.i(TAG, "BlugluDeviceManager::cancel()");

        if (callback != null)
            m_callback = callback;

        m_Brand = "";
        m_DeviceName = "";
        m_SerialNumber = "";

        synchronized (BlugluDeviceManager.this)
        {
            // Stop the Bluglu service
            if (m_Bluglu != null)
                m_Bluglu.stop();
            if (D)
                Log.i(TAG, "--- Bluglu service stopped. ---");
        }

                if (m_workingThread != null && !Thread.interrupted())
                {
                    m_workingThread.interrupt();
                    m_workingThread = null;
                }
    }

    @SuppressWarnings("unused")
    private void updateState(String state)
    {
        try
        {
            statePayload.put("state", state);
        }
        catch (JSONException e)
        {
            returnError("Error when convert connection state");
        }
        returnResult(ResultFactory.createResult(PluginResult.Status.OK, statePayload));
    }

    private void prepareData()
    {

        if (payload.has(PAYLOAD_BATTERY_STATUS_PERCENTAGE) && payload.has(PAYLOAD_READING)
                    && payload.has(PAYLOAD_TIME) && payload.has(PAYLOAD_SERIAL) && payload.has(PAYLOAD_BRAND))
        {
            try
            {
                payload.put(PAYLOAD_TYPE, "blood glucose");
            }
            catch (JSONException e)
            {
                returnError("Error when convert " + PAYLOAD_TYPE);
            }

            try
            {
                payload.put(PAYLOAD_SYSTEM, "mmol/L");
            }
            catch (JSONException e)
            {
                returnError("Error when convert " + PAYLOAD_SYSTEM);
            }

            returnResult(ResultFactory.createResult(PluginResult.Status.OK, payload));
        }
    }

    private void prepareData(String key, float value)
    {
        try
        {
            payload.put(key, value);
        }
        catch (JSONException e)
        {
            returnError("Error when convert " + key);
        }

        prepareData();
    }

    private void prepareData(String key, String value)
    {
        try
        {
            payload.put(key, value);
        }
        catch (JSONException e)
        {
            returnError("Error when convert " + key);
        }

        prepareData();
    }

    private void prepareData(String key, int value)
    {
        try
        {
            payload.put(key, value);
        }
        catch (JSONException e)
        {
            returnError("Error when convert " + key);
        }

        prepareData();
    }

    protected void returnResult(final PluginResult result)
    {
        m_plugin.returnResult(result, m_callback);
    }

    private void returnError(String msg)
    {
        returnResult(ResultFactory.createResult(PluginResult.Status.ERROR, msg));
    }
}