/* This file is automatically copied form (NexJ PhoneGap) DevicePlugin project.
 * Do not modify this file -- YOUR CHANGES WILL BE ERASED! 
 */ 
package com.nexj.bluetooth;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.FutureTask;
import java.util.concurrent.ThreadPoolExecutor;

import com.nexj.pseudo.phonegap.CallbackContext;
import com.nexj.pseudo.phonegap.CordovaInterface;
import com.nexj.pseudo.phonegap.PluginResult;
import org.json.JSONArray;
import org.json.JSONObject;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;

import com.nexj.bluetooth.util.BondedDevicesCommand;
import com.nexj.bluetooth.util.ConnectCommand;
import com.nexj.bluetooth.util.DeviceStartCommand;
import com.nexj.bluetooth.util.DeviceStopCommand;
import com.nexj.bluetooth.util.DisableCommand;
import com.nexj.bluetooth.util.DiscoverableCommand;
import com.nexj.bluetooth.util.EnableCommand;
import com.nexj.bluetooth.util.IsEnabledCommand;
import com.nexj.bluetooth.util.IsSupportedCommand;
import com.nexj.bluetooth.util.ResultFactory;
import com.nexj.bluetooth.util.StopCommand;
import com.nexj.bluetooth.util.TypesCommand;

public class MedicalDevicePlugin
{
    public static final String ACTION_ENABLE = "enable";
    public static final String ACTION_DISABLE = "disable";
    public static final String ACTION_REQUEST_DISCOVERABLE = "discoverable";
    public static final String ACTION_GETBONDEDDEVICES = "getBonded";
    public static final String ACTION_IS_ENABLED = "isEnabled";
    public static final String ACTION_IS_SUPPORTED = "isSupported";
    public static final String ACTION_GET_TYPES = "types";
    public static final String ACTION_CONNECT = "connect";
    public static final String ACTION_DISCONNECT = "disconnect";

    CordovaInterface m_cordova;
    public final Activity m_activity;
    protected final ExecutorService m_threadPool;

    protected final BluetoothAndroid m_bluetoothAndroid;
    protected final BroadcastReceiver m_broadcastReceiver;
    protected final List<FutureTask<Runnable>> m_interruptableTasks;
    protected final Map<String, Vector<Runnable>> m_actionCallbackMap;
    protected final Map<Runnable, Integer> m_actionStateMap;

    protected final Map<String, DeviceManager> m_deviceMap;

    public MedicalDevicePlugin(CordovaInterface cordova)
    {
        m_cordova = cordova;
        m_activity = cordova.getActivity();
        m_threadPool = cordova.getThreadPool();

        m_interruptableTasks = new Vector<FutureTask<Runnable>>(16);
        m_broadcastReceiver = new BluetoothBroadcastReceiver();
        m_actionCallbackMap = new ConcurrentHashMap<String, Vector<Runnable>>(8);
        m_actionStateMap = new ConcurrentHashMap<Runnable, Integer>(8);
        m_deviceMap = new HashMap<String, DeviceManager>(DeviceManager.SUPPORTED_BRANDS_TYPES.length());

        m_bluetoothAndroid = new BluetoothAndroid();
        m_deviceMap.put("AND", new ANDBluetoothServer());
        m_deviceMap.put("BluGlu", new BlugluDeviceManager(this));
        m_deviceMap.put("Polytel", new PolytelDeviceManager(this));

        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
        intentFilter.addAction(BluetoothAdapter.ACTION_SCAN_MODE_CHANGED);

        m_activity.registerReceiver(m_broadcastReceiver, intentFilter);
    }

    public void onDestroy()
    {
        m_activity.unregisterReceiver(m_broadcastReceiver);

        Iterator<Entry<String, DeviceManager>> itr = m_deviceMap.entrySet().iterator();

        while (itr.hasNext())
        {
            DeviceManager adapter = itr.next().getValue();

            adapter._stop(null);
        }

        new StopCommand().run();
    }

    public void registerCommandForAction(String action, Runnable command, int state)
    {
        Vector<Runnable> callbackList = m_actionCallbackMap.get(action);

        if (callbackList == null)
        {
            callbackList = new Vector<Runnable>(8);
        }

        callbackList.add(command);

        m_actionCallbackMap.put(action, callbackList);

        if (state > -1)
        {
            m_actionStateMap.put(command, Integer.valueOf(state));
        }
    }

    public boolean execute(String action, JSONArray args, CallbackContext callbackContext)
    {
        if (ACTION_CONNECT.equals(action))
        {
            JSONObject params = args.optJSONObject(0);
            String brand = params.optString("brand");

            if (m_deviceMap.containsKey(brand))
            {
                if ("AND".equals(brand))
                {
                    execute(new ConnectCommand(this, m_deviceMap.get(brand), callbackContext, ResultFactory.JSONToMap(params)), true);
                }
                else
                {
                    execute(new DeviceStartCommand(this, m_deviceMap.get(brand), callbackContext, ResultFactory.JSONToMap(params)));
                }
            }
            else
            {
                return false;
            }
        }
        else if (ACTION_DISCONNECT.equals(action))
        {
            JSONObject params = args.optJSONObject(0);
            String brand = params.optString("brand");

            if (m_deviceMap.containsKey(brand))
            {
                execute(new DeviceStopCommand(this, m_deviceMap.get(brand), callbackContext, m_deviceMap));
            }
            else
            {
                return false;
            }
        }
        else if (ACTION_ENABLE.equals(action))
        {
            execute(new EnableCommand(this, callbackContext));
        }
        else if (ACTION_DISABLE.equals(action))
        {
            execute(new DisableCommand(this, callbackContext));
        }
        else if (ACTION_REQUEST_DISCOVERABLE.equals(action))
        {
            execute(new DiscoverableCommand(this, callbackContext, args.optInt(0)));
        }
        else if (ACTION_IS_SUPPORTED.equals(action))
        {
            execute(new IsSupportedCommand(this, callbackContext));
        }
        else if (ACTION_GETBONDEDDEVICES.equals(action))
        {
            // TODO: this is a hack
            execute(new BondedDevicesCommand(this, callbackContext));
        }
        else if (ACTION_IS_ENABLED.equals(action))
        {
            execute(new IsEnabledCommand(this, callbackContext));
        }
        else if (ACTION_GET_TYPES.equals(action))
        {
            execute(new TypesCommand(this, callbackContext));
        }
        else
        {
            return false;
        }

        return true;
    }

    public void returnResult(final PluginResult result, final CallbackContext callbackContext)
    {
        m_activity.runOnUiThread(new Runnable()
        {
            public void run()
            {
                callbackContext.sendPluginResult(result);
            }
        });
    }

    public void execute(final Runnable command)
    {
        execute(command, false);
    }

    public void execute(final Runnable command, boolean interruptable)
    {
        if (!m_threadPool.isShutdown())
        {
            if (interruptable)
            {
                FutureTask<Runnable> task = new FutureTask<Runnable>(command, null)
                            {
                    protected void done()
                    {
                        m_interruptableTasks.remove(this);
                    }
                            };

                            m_interruptableTasks.add(task);

                            m_threadPool.execute(task);
            }
            else
            {
                m_threadPool.execute(command);
            }
        }
    }

    public void interrupt()
    {
        Iterator<FutureTask<Runnable>> itr = m_interruptableTasks.iterator();

        while (itr.hasNext())
        {
            FutureTask<Runnable> task = itr.next();

            if (!task.isDone())
            {
                task.cancel(true);
            }
        }

        if (!m_threadPool.isShutdown())
        {
            if (m_threadPool instanceof ThreadPoolExecutor)
            {
                ((ThreadPoolExecutor)m_threadPool).purge();
            }
        }

        m_interruptableTasks.clear();
    }

    protected class BluetoothBroadcastReceiver extends BroadcastReceiver
    {
        public void onReceive(Context context, Intent intent)
        {
            String action;
            Vector<Runnable> callbackList;

            if ((action = intent.getAction()) == null
                        || (callbackList = m_actionCallbackMap.get(action)) == null
                        || callbackList.isEmpty())
            {
                return;
            }

            Iterator<Runnable> itr = callbackList.iterator();

            while (itr.hasNext())
            {
                Runnable command = itr.next();
                Integer state;

                if ((state = m_actionStateMap.get(command)) == null)
                {
                    continue;
                }

                if (BluetoothAdapter.ACTION_SCAN_MODE_CHANGED.equals(action))
                {
                    int curr = intent.getIntExtra(BluetoothAdapter.EXTRA_SCAN_MODE, -1);

                    if ((BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE == intent.getIntExtra(BluetoothAdapter.EXTRA_PREVIOUS_SCAN_MODE, -1)
                                || BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE == curr)
                                && state.intValue() == curr)
                    {
                        execute(command);
                    }
                }
                else if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(action))
                {
                    if (state.intValue() == intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, -1))
                    {
                        execute(command);
                    }
                }
            }
        }
    }

    public PluginResult execute(String arg0, JSONArray arg1, String arg2)
    {
        // TODO Auto-generated method stub
        return null;
    }
}
