/* This file is automatically copied form (NexJ PhoneGap) DevicePlugin project.
 * Do not modify this file -- YOUR CHANGES WILL BE ERASED! 
 */ 
package com.nexj.bluetooth;

import java.util.HashMap;

import com.nexj.pseudo.phonegap.CallbackContext;
import com.nexj.pseudo.phonegap.PluginResult;
import org.json.JSONException;
import org.json.JSONObject;

import polymap.base.Polymap;
import polymap.base.PolymapDeviceData;
import polymap.base.PolymapMeasurementDataGma;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import com.nexj.bluetooth.util.ResultFactory;

public class PolytelDeviceManager extends DeviceManager
{
    // Debugging
    private static final String TAG = "PolytelDevoceManager";
    private static final boolean D = false;

    // Message types sent from the Polytel Handler
    private int measurementDisplayGroup = 0;
    @SuppressWarnings("unused")
    private int maximumMeasurementDisplayGroup = 0;
    private final int MEASUREMENTS_PER_GROUP = 50;

    // Keys of pass-back payload
    public static final String PAYLOAD_TYPE = "type";
    public static final String PAYLOAD_BRAND = "brand";
    public static final String PAYLOAD_TIME = "time";
    public static final String PAYLOAD_READING = "reading";
    public static final String PAYLOAD_SYSTEM = "system";
    public static final String PAYLOAD_SERIAL = "serial";
    public static final String PAYLOAD_BATTERY_STATUS_PERCENTAGE = "battery status(%)";

    //private Handler and working thread for receiving data
    protected Thread m_workingThread;
    protected static Handler m_messageHandler;

    // Members for persisting passed-in parameters
    protected final MedicalDevicePlugin m_plugin;

    protected CallbackContext m_callback;
    //protected String m_CallbackId; // may be removed
    protected String m_Brand = "";
    protected String m_DeviceType = "";
    protected String m_SerialNumber = "";

    // Member object for the Polytel service
    private Polymap m_Polymap = null;
    private PolymapDeviceData mDeviceData = null;

    public PolytelDeviceManager(MedicalDevicePlugin plugin)
    {
        // If the adapter is null, then Bluetooth is not supported
        if (BluetoothAndroid.BLUETOOTH_ADAPTER == null)
        {
            Log.e(TAG, "+++ Bluetooth is not available +++");
        }

        m_plugin = plugin;

        if (D)
            Log.i(TAG, "+++ CREATING PolytelManager +++");
    }

    @Override
    public void _start(HashMap<String, Object> params, CallbackContext callback)
    {
        m_Brand = (String)params.get("brand");
        m_DeviceType = (String)params.get("type");
        m_SerialNumber = (String)params.get("serial");
        m_callback = callback;

        if (D)
        {
            Log.i(TAG, "+++ Passed-in brand: " + m_Brand);
            Log.i(TAG, "+++ Passed-in type: " + m_DeviceType);
            Log.i(TAG, "+++ Passed-in serial number: [" + m_SerialNumber + "]");
        }

        m_workingThread = new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                Looper.prepare();

                m_messageHandler = new Handler()
                {
                    public void handleMessage(Message msg)
                    {
                        switch (msg.what)
                        {
                            case Polymap.MSG_RETRIEVAL_SUCCESS:
                                mDeviceData = m_Polymap.getDeviceData();

                                measurementDisplayGroup = 0;
                                maximumMeasurementDisplayGroup = (mDeviceData.measurements.size() - 1) / MEASUREMENTS_PER_GROUP;

                                prepareData();
                                break;

                            case Polymap.MSG_RETRIEVAL_ERROR:
                                returnResult(new PluginResult(PluginResult.Status.OK, "ERROR when retriving data"));
                                break;
                        }
                    }
                };

                // If BT is not on, request that it be enabled.
                if (!BluetoothAndroid.isEnabled())
                {
                    returnError("Bluetooth is not enabled");
                }
                else
                {
                    if (m_Polymap == null)
                        setupDevice();
                    if (D)
                        Log.e(TAG, "++ Setup Polytel ++");
                }

                if (m_Polymap != null)
                {
                    // Start listen to Polytel devices
                    m_Polymap.acceptSlaveData();

                    if (D)
                        Log.e(TAG, "++ Polymap started (listening) ! ++");
                }

                Looper.loop();
            }
        });

        m_workingThread.start();
    }

    @Override
    public void _stop(CallbackContext callback)
    {
        if (D)
            Log.i(TAG, "PolytelDeviceManager::cancel()");

        if (callback != null)
            m_callback = callback;

        m_Brand = "";
        m_DeviceType = "";
        m_SerialNumber = "";

        synchronized (PolytelDeviceManager.this)
        {
            // Stop listen to Polytel devices
            if (m_Polymap != null)
                m_Polymap.stopAcceptingRemoteDeviceData();
            if (D)
                Log.i(TAG, "--- Polytel service stopped. ---");
        }

        if (m_workingThread != null && !Thread.interrupted())
        {
            m_workingThread.interrupt();
            m_workingThread = null;
        }
    }

    private void setupDevice()
    {
        Log.d(TAG, "setupDevice()");

        // Initialize Polytel service
        m_Polymap = new Polymap(m_messageHandler, (Context)m_plugin.m_cordova);
        m_Polymap.forceGmaRetry = false;

        UncaughtExceptionHandler uncaughtExpectionHandler = new UncaughtExceptionHandler((Context)m_plugin.m_cordova);
        Thread.setDefaultUncaughtExceptionHandler(uncaughtExpectionHandler);
    }

    private void prepareData()
    {
        int startIndex = 0;
        int endIndex = 0;
        PolymapMeasurementDataGma mGma;

        if (mDeviceData.measurements.size() > 0)
        {
            String device = mDeviceData.deviceType;
            String serial = mDeviceData.serialNumberDevice;
            Float batteryStatus = mDeviceData.batteryVoltageGMA / 6;

            // Displaying each measurement
            startIndex = measurementDisplayGroup * MEASUREMENTS_PER_GROUP;
            endIndex = mDeviceData.measurements.size() < startIndex + MEASUREMENTS_PER_GROUP ?
                        mDeviceData.measurements.size() : startIndex + MEASUREMENTS_PER_GROUP;

            //Display units: note that meters may use either mg/dL or mmol/L for display ("display units"), but always transmit mg/dL (units)
            for (int i = startIndex; i < endIndex; i++)
            {
                if (mDeviceData.deviceClass == Polymap.DEVICE_CLASS_GMA)
                {
                    mGma = (PolymapMeasurementDataGma)mDeviceData.measurements.get(i);

                    JSONObject tmp = new JSONObject();
                    try
                    {
                        tmp.put(PAYLOAD_TYPE, "blood glucose");
                        tmp.put(PAYLOAD_BRAND, "Polytel");
                        tmp.put("meter", device);
                        tmp.put(PAYLOAD_TIME, mGma.timeStamp_METER_LOCAL.getTimeInMillis() / 1000L);

                        if (mGma.units.indexOf("mg") >= 0 || mGma.units.indexOf("MG") >= 0)
                        {
                            tmp.put(PAYLOAD_READING, mGma.value * 0.0555);
                        }
                        else
                        {
                            tmp.put(PAYLOAD_READING, mGma.value);
                        }
                        tmp.put(PAYLOAD_SYSTEM, "mmol/L");
                        tmp.put(PAYLOAD_SERIAL, serial);
                        tmp.put(PAYLOAD_BATTERY_STATUS_PERCENTAGE, batteryStatus);

                        // extra info
                        //tmp.put("corrected time", mGma.correctedTimeStamp_LOCAL.getTimeInMillis() / 1000L);
                        //tmp.put("measurement type", mGma.measurementType);
                        //tmp.put("mealFlag", mGma.mealFlag);
                        //tmp.put("mealComment", mGma.mealComment);
                        //tmp.put("battery_lowFlag", mGma.battery_lowFlag);
                        //tmp.put("overflowFlag", mGma.overflowFlag);
                        //tmp.put("underflowFlag", mGma.underflowFlag);
                        //tmp.put("tempOutOfRangeFlag", mGma.tempOutOfRangeFlag);
                        //tmp.put("parityError", mGma.parityError);
                    }
                    catch (JSONException e)
                    {
                        e.printStackTrace();
                    }

                    if (m_SerialNumber.equals("") || m_SerialNumber.equals(serial))
                        returnResult(new PluginResult(PluginResult.Status.OK, tmp));
                }
            }
        }
    }


    protected void returnResult(final PluginResult result)
    {
        m_plugin.returnResult(result, m_callback);
    }

    private void returnError(String msg)
    {
        returnResult(ResultFactory.createResult(PluginResult.Status.ERROR, msg));
    }
}

class UncaughtExceptionHandler implements Thread.UncaughtExceptionHandler
{
    public UncaughtExceptionHandler(Context context)
    {
        Log.d("Polymap", "Application uncaught exception handler constructed");
    }

    public void uncaughtException(Thread t, Throwable e)
    {
        Log.d("Polymap", "Application uncaught exception");
    }
}